import { app } from "./app";
import { APIException, NetworkConnectionException, ReactLifecycleException, RuntimeException } from "./Exception";
import { loggerContext } from "./platform/loggerContext";
export class LoggerImpl {
    constructor() {
        this.environmentContext = {};
        this.logQueue = [];
        this.environmentContext = loggerContext;
    }
    addContext(context) {
        this.environmentContext = { ...this.environmentContext, ...context };
    }
    info(action, info) {
        return this.appendLog("OK", { action, info: info || {} });
    }
    warn(data) {
        return this.appendLog("WARN", data);
    }
    error(data) {
        return this.appendLog("ERROR", data);
    }
    exception(exception, action) {
        if (exception instanceof NetworkConnectionException) {
            const info = { url: exception.requestURL };
            return this.appendLog("WARN", { action, errorCode: "NETWORK_FAILURE", errorMessage: exception.message, info });
        }
        else {
            const info = {};
            let isWarning = false;
            let errorCode = "OTHER_ERROR";
            if (exception instanceof APIException) {
                errorCode = `API_ERROR_${exception.statusCode}`;
                // Following cases are treated as expected
                if ([401, 403, 426, 503].includes(exception.statusCode)) {
                    isWarning = true;
                }
                else if (exception.statusCode === 400) {
                    if (exception.errorCode === "VALIDATION_ERROR") {
                        errorCode = "API_VALIDATION_FAIL";
                    }
                    else {
                        isWarning = true;
                    }
                }
                info.requestURL = exception.requestURL;
                if (exception.errorCode) {
                    info.errorCode = exception.errorCode;
                }
                if (exception.errorId) {
                    info.errorId = exception.errorId;
                }
            }
            else if (exception instanceof ReactLifecycleException) {
                errorCode = "LIFECYCLE_ERROR";
                info.stackTrace = exception.componentStack;
                info.appState = JSON.stringify(app.store.getState().app);
            }
            else if (exception instanceof RuntimeException) {
                errorCode = "JS_ERROR";
                info.errorObject = JSON.stringify(exception.errorObject);
                info.appState = JSON.stringify(app.store.getState().app);
            }
            return this.appendLog(isWarning ? "WARN" : "ERROR", { action, errorCode, errorMessage: exception.message, info });
        }
    }
    collect() {
        return this.logQueue;
    }
    empty() {
        this.logQueue = [];
    }
    appendLog(result, data) {
        const completeContext = {};
        Object.entries(this.environmentContext).map(([key, value]) => {
            completeContext[key] = typeof value === "string" ? value : value();
        });
        const event = {
            date: new Date(),
            result,
            context: completeContext,
            elapsedTime: 0,
            ...data,
        };
        this.logQueue.push(event);
        return () => {
            event.elapsedTime = new Date().getTime() - event.date.getTime();
        };
    }
}
//# sourceMappingURL=Logger.js.map